#############################################################
#
# Any custom stuff you feel like doing....
#
#############################################################
CUST_DIR:=package/customize/source

custom_file_list := $(shell find $(CUST_DIR) -not '(' -type d -name CVS -prune ')' -type f)

custom_installed_file_list := $(custom_file_list:$(CUST_DIR)/%=$(TARGET_DIR)/%)

NETKIT_VERSION := netkit-rsh-0.17
NETKIT_DIR     := $(BUILD_DIR)/$(NETKIT_VERSION)
NETKIT_TARBALL := $(NETKIT_VERSION).tar.gz
NETKIT_APPS    :=  rlogind/rlogind rcp/rcp rshd/rshd
NETKIT_BUILT_APPS     := $(addprefix $(NETKIT_DIR)/, $(NETKIT_APPS))

OPENSSL_VERSION := openssl-0.9.7e
OPENSSL_DIR	:= $(BUILD_DIR)/$(OPENSSL_VERSION)
OPENSSL_TARBALL := $(OPENSSL_VERSION).tar.gz
OPENSSL_APPS	:= apps/openssl
OPENSSL_BUILT_APPS := $(addprefix $(OPENSSL_DIR)/, $(OPENSSL_APPS))

INSTALL_DIR    := $(TARGET_DIR)/bin
INSTALLED_APPS := $(addprefix $(INSTALL_DIR)/, $(notdir $(NETKIT_APPS))) \
		  $(addprefix $(INSTALL_DIR)/, $(notdir $(OPENSSL_APPS)))

$(custom_installed_file_list): $(TARGET_DIR)/%: $(CUST_DIR)/% FORCE
	@if [ ! -d $(@D) ]; then echo mkdir -p $(@D); mkdir -p $(@D); fi
	cp -df --preserve=mode --preserve=ownership $< $@

customize: $(custom_installed_file_list) $(INSTALLED_APPS)

#############################################################
# netkit-rsh
#############################################################
netkit: $(INSTALLED_APPS)

$(NETKIT_BUILT_APPS): $(BUILD_DIR)/.f
	tar xzf $(DL_DIR)/$(NETKIT_TARBALL) -C $(BUILD_DIR)
	(cd $(NETKIT_DIR) ; \
	 patch -p1 <$(DL_DIR)/../patches/$(NETKIT_VERSION).patch ;\
	 CC=$(TARGET_CC) ./configure ;\
	 make ;\
	)

$(INSTALL_DIR)/rlogind: $(NETKIT_DIR)/rlogind/rlogind $(INSTALL_DIR)/.f
	cp -f $(filter-out %/.f,$^) $@

$(INSTALL_DIR)/rshd: $(NETKIT_DIR)/rshd/rshd $(INSTALL_DIR)/.f
	cp -f $(filter-out %/.f,$^) $@

$(INSTALL_DIR)/rcp: $(NETKIT_DIR)/rcp/rcp $(INSTALL_DIR)/.f
	cp -f $(filter-out %/.f,$^) $@

#############################################################
# openssl
#############################################################
$(OPENSSL_BUILT_APPS): $(BUILD_DIR)/.f
	tar xzf $(DL_DIR)/$(OPENSSL_TARBALL) -C $(BUILD_DIR)
	( cd $(OPENSSL_DIR) ; \
	  ./Configure --prefix=/ --openssldir=/lib/openssl linux-mipsel ; \
	  make ; \
	)

$(INSTALL_DIR)/openssl: $(OPENSSL_DIR)/apps/openssl $(INSTALL_DIR)/.f
	cp -f $(filter-out %/.f,$^) $@

%/.f:
	mkdir -p $(dir $@)
	touch $@

#############################################################
#
# Toplevel Makefile options
#
#############################################################
ifeq ($(strip $(BR2_PACKAGE_CUSTOMIZE)),y)
TARGETS+=customize
endif
