#!/bin/sh
#
# Verify and correct XENV mirroring
#
# WARNING: The addresses for the XENV blocks are hard-coded below.
#

XENV_PRIM=/dev/mtdblock/1	# primary XENV block
XENV_MIRR=/dev/mtdblock/2	# secondary (mirror) XENV block

start() {
    echo -n "Checking XENV ... "
    PRIM_CHK=`genxenv g $XENV_PRIM p.xenv_mirror 2>/dev/null | sed -e s/\"//g`
    MIRR_CHK=`genxenv g $XENV_MIRR p.xenv_mirror 2>/dev/null | sed -e s/\"//g`

    if [ "$PRIM_CHK" != "yes" ] && [ "$MIRR_CHK" != "yes" ]
    then
	echo "mirroring not enabled"
	exit 0
    fi

    # Search the bootlog to find the XENV address
    # and then compare it against the expected locations of the
    # XENV tables.
    eval $(dmesg | grep XENV_ADDR)
    if [ "$XENV_ADDR" = "0x48000000" ]
    then
        # using the primary XENV table make sure the mirror
        # is up-to-date

        xenv_prim_md5=`dd if=$XENV_PRIM | md5sum`
        xenv_mirr_md5=`dd if=$XENV_MIRR | md5sum`

        if [ "$xenv_prim_md5" != "$xenv_mirr_md5" ]
        then
            # Mirror is out of sync with primary XENV block
            # fix it
            dd if=$XENV_PRIM of=$XENV_MIRR
	    echo "copied primary to mirror"
	else
	    echo "mirror matches primary"
        fi
    elif [ "$XENV_ADDR" = "0x48020000" ]
    then
        # using the backup XENV table
        # the first XENV block must be invalid, restore it from the mirror
        dd if=$XENV_MIRR of=$XENV_PRIM
	echo "copied mirror to primary"
    else
        # can not determine which table is in use
        echo "WARNING: XENV mirroring enabled, but could not identify XENV block."
        echo "         XENV_ADDR=$XENV_ADDR"
    fi

    echo "  DONE."
}	

stop() {
    echo "Stop is not implemented."
}
restart() {
	stop
	start
}	

case "$1" in
  start)
  	start
	;;
  stop)
  	stop
	;;
  restart|reload)
  	restart
	;;
  *)
	echo $"Usage: $0 {start|stop|restart}"
	exit 1
esac

exit $?

