#!/bin/sh
#
# update script for upgrade/downgrade operation.
# Finds update data stream, extracts it into a temporary directory,
# and hands control to update.sh.
#
# $URL: http://svn.sj.pioneer-pra.com/targets/smp8634/trunk/patches/smp86xx_rootfs/target_skeleton/etc/init.d/S30updateMode.sh $
# $Revision: 54322 $
# $Date: 2006-11-10 14:24:05 -0800 (Fri, 10 Nov 2006) $
# $Author: rkwan $

# constants
XENV_BLOCK=/dev/mtdblock/1
THIS=`basename $0`    # script identifier for messages

# global variables
RETCODE=fail    # only mark 'succ' on DVD update.sh success

start() {
    updateKey=`genxenv g $XENV_BLOCK p.updateKey 2>/dev/null`
    # echo DBG updateKey is $updateKey
    if [ -n "$updateKey" ]
    then
        board_id=`genxenv g $XENV_BLOCK a.board_id 2>/dev/null`
        echo $THIS: board_id $board_id
        case "$board_id" in
          \"BP06*\") tc=P; OEMBDR=n ;;
          \"OEMBDP*\") tc=S; OEMBDR=n ;;
          \"OEMBDR*\") tc=R; OEMBDR=y;;
          *) tc=X; BDR=n ;;
        esac

        # load front panel helper script functions
        source /etc/init.d/fip.sh

	# determine update data stream command (dscmd)
	altSource=`genxenv g $XENV_BLOCK p.altSource 2>/dev/null`
        echo $THIS: altSource $altSource
	if [ -n "$altSource" ]
	then
	    dsprep=
	    dspost=
	    dsvalid="true"
            notifier=
            notifier_static=
            dwldmsg1=
            dwldmsg2=

	    set `echo $altSource | sed -e 's/:/ /' -e 's/"//g'`
	    type=$1
	    param=$2
	    case "$type" in
	      net)      # web server
                dscmd="wget -O- $param"
                notifier=fip_animate
                notifier_static=fip_static_msg
                dwldmsg1="NET DWLD"
                dwldmsg2="NET-DWLD"
                fip_animate $dwldmsg1 $dwldmsg2
                # verify that the network is available
                /sbin/ifconfig eth0 | grep UP >/dev/null 2>&1
                if [ $? -eq 0 ]
                then
                    # and verify that we have an ip address
                    # allow up-to 30 seconds for udhcpc to get the
                    # address
                    echo "Checking for IP address."
                    i=30
                    until /sbin/ifconfig eth0 | grep "inet addr:"
                    do
                        sleep 1
                        let i=i-1
                        if [ $i -le 0 ]
                        then
                            echo "No IP address! Network update impossible"
                            fip_static_msg 'WRT FAILED'
                            genxenv d $XENV_BLOCK p.updateKey 2>/dev/null
                            genxenv d $XENV_BLOCK p.altSource 2>/dev/null
                            while true
                            do
                                sleep 60
                            done
                            # never exit
                        fi
                    done
                else
                    echo "Network down. Network update impossible"
                    fip_static_msg 'WRT FAILED'
                    genxenv d $XENV_BLOCK p.updateKey 2>/dev/null
                    genxenv d $XENV_BLOCK p.altSource 2>/dev/null
                    while true
                    do
                        sleep 60
                    done
                    # never exit
                fi
                ;;
	      glue)     # glue logic
                dscmd="pwr_cntl 207"
                notifier=echo
                notifier_static=echo
                dwldmsg1="GLUE DWLD"
                dwldmsg2="GLUE-DWLD"
                ;;
	      dvd)	# different location on DVD
                dsprep="pwr_cntl 208; mount /dvd"
                dscmd="cat $param"
		dspost="umount /dvd; eject"
                dsvalid="-e /dvd/$param"
                notifier=fip_animate
                notifier_static=fip_static_msg
                dwldmsg1="DISC DWLD"
                dwldmsg2="DISC-DWLD"
                ;;
	      *)  echo Unparsable update operation.  Update failed.
                  fip_static_msg 'WRT FAILED'
                  genxenv d $XENV_BLOCK p.updateKey 2>/dev/null
                  genxenv d $XENV_BLOCK p.altSource 2>/dev/null
                  while true
                  do
                      sleep 60
                  done
		  # never exit
                  ;;
	    esac
	else		# default DVD location
	    dsprep="pwr_cntl 208; mount /dvd"
	    dscmd="cat /dvd/${tc}0002.BIN"
	    dspost="umount /dvd; sleep 1; eject"
	    dsvalid="-e /dvd/${tc}0002.BIN"
            notifier=fip_animate
            notifier_static=fip_static_msg
            dwldmsg1="DISC DWLD"
            dwldmsg2="DISC-DWLD"
	fi

	# prepare for stream, e.g., mount DVD
	eval $dsprep

	# no DVD, wrong DVD, or bad path
	# covers case of: updateKey exists, but DVD update failed,
	# e.g., power outage, other weird interruptions
	if [ ! "$dsvalid" ] 
	then
	    eval $dspost
	    return 1
	fi

        # display download message on front panel.
        $notifier "$dwldmsg1" "$dwldmsg2"

        ####
        # prepare workspace
        [ -e /tmp/update ] || mkdir /tmp/update
        mount -t tmpfs -o size=70M tmpfs /tmp/update
        # Get multi-line key into file with proper newlines.
        touch /tmp/update/key
        for line in $updateKey
        do echo $line >> /tmp/update/key
        done
        # cat /tmp/update/key        # for debugging

        # decrypt and extract to /tmp/update
        cd /tmp/update
        # mount /dvd
	if [ "$dsvalid" ] 
        then 
            eval $dscmd | \
		openssl enc -d -aes-256-cbc -salt \
                -pass file:/tmp/update/key | \
                tar -xzf -              # tar -tvzf -
        else
            echo $THIS ERROR: File $update_file not on DVD.
	    umount /tmp/update
            $notifier_static "WRT FAILED"
            genxenv d $XENV_BLOCK p.updateKey 2>/dev/null
            genxenv d $XENV_BLOCK p.altSource 2>/dev/null
            while true
            do
                sleep 60
            done
            # never exit
        fi

        # for DVD, unmount and eject
        eval $dspost

        # execute update script
        if [ -e /tmp/update/clrtxt/update.sh ]
        then
            cd /tmp/update/clrtxt; exec ./update.sh
	else
	    echo $This ERROR: missing update.sh 
            $notifier_static "WRT FAILED"
            genxenv d $XENV_BLOCK p.updateKey 2>/dev/null
            genxenv d $XENV_BLOCK p.altSource 2>/dev/null
            while true
            do
                sleep 60
            done
            # never exit
        fi
	while true
	do  echo "$THIS ERROR: Should not get here!"
            $notifier_static "WRT FAILED"
            genxenv d $XENV_BLOCK p.updateKey 2>/dev/null
            genxenv d $XENV_BLOCK p.altSource 2>/dev/null
            while true
            do
                sleep 60
            done
            # never exit
	done

    fi
}

stop() {
    cd /mnt
    umount /tmp/update
    rmdir /tmp/update
}

restart() {
	stop
	start
}	

case "$1" in
  start)
  	start
	;;
  stop)
  	stop
	;;
  restart|reload)
  	restart
	;;
  *)
	echo $"Usage: $0 {start|stop|restart}"
	exit 1
esac

exit $?

