#
# fip.sh - shell functions to manipulate the front panel
#
# $URL: http://svn.sj.pioneer-pra.com/targets/smp8634/trunk/patches/smp86xx_rootfs/target_skeleton/etc/init.d/fip.sh $
# $Rev: 48233 $
# $Date: 2006-07-20 15:27:25 -0700 (Thu, 20 Jul 2006) $
# $Author: htrieu $
#
# This file should be sourced into a script.
#
# There are 4 functions:
#   fip_animate        - the arguments to this function are display in a
#                        loop with a one second delay between arguments
#   fip_stop_animation
#   fip_start_animation
#   fip_static_msg      - stops animation loop and displays a static
#                         message

if [ -n "$fipuser" -a -e "$fipuser" ]
then
    echo fipuser is $fipuser
elif [ -e /bin/fipuser ]
then
    fipuser=/bin/fipuser
elif [ -e /mnt/workdir/bin/fipuser ]
then
    fipuser=/mnt/workdir/bin/fipuser
else
    fipuser=echo
fi

fip_pid_file=/tmp/fip_pid
msg_file=/tmp/messages.txt

function fip() {
    op=$1; shift
    echo fip $*

    case $op in
      anim_stop)  fip_stop_animation ;;
      anim_start) fip_start_animation $* ;;
      animate)    fip_animate "$1" "$2" ;;
      static_msg) fip_static_msg "$*" ;;
    esac
}

# fip_stop_animation
function fip_stop_animation() {
    if [ -e "$fip_pid_file" ]
    then
        set $(cat $fip_pid_file)
        kill $1
        rm $fip_pid_file
        sleep 1
    fi
}

# fip_start_animation
function fip_start_animation() {
    if [ ! -e $fip_pid_file ]
    then
        if [ ! -e $msg_file ]
        then
            touch $msg_file
        fi
        (
            while true
            do
                while read A
                do
                    if [ "$A" = "stop" ]
                    then
                        $fipuser ""
                        exit
                    fi
                    $fipuser "$A"
                    sleep 1
                done < $msg_file
            done
        ) &

        echo "$!" > $fip_pid_file
    else
        echo "fip animation is running..."
    fi
}

# fip_animate [<message> ...]
# Each argument is displayed for 1 second. The list loops until replaced
# or stopped.
function fip_animate() {
    fip_stop_animation
    echo $1 >$msg_file
    while shift
    do
        if [ "$1" != "" ]
        then
            echo $1 >>$msg_file
        fi
    done
    fip_start_animation
}

# static_msg <message>
# Animation, if any is stopped and the first argument is displayed.
function fip_static_msg() {
    fip_stop_animation
    $fipuser "$1"
}


